/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "packetqueue.h"
#include "assert/assert.h"
#include "util/memory.h"

void packetQueueInit(packetqueue_t *queue) {
  assertNotNull(queue, "Packet queue is NULL");
  memoryZero(queue, sizeof(packetqueue_t));
}

void packetQueuePushIn(packetqueue_t *queue, const packet_t *packet) {
  assertNotNull(queue, "Packet queue is NULL");
  assertNotNull(packet, "Packet is NULL");
  assertTrue(
    queue->packetsInCount < PACKET_QUEUE_MAX_SIZE, "Inbound packet queue is full"
  );

  queue->packetsIn[queue->packetsInCount++] = *packet;
}

void packetQueuePushOut(packetqueue_t *queue, const packet_t *packet) {
  assertNotNull(queue, "Packet queue is NULL");
  assertNotNull(packet, "Packet is NULL");
  assertTrue(
    queue->packetsOutCount < PACKET_QUEUE_MAX_SIZE, "Outbound packet queue is full"
  );

  queue->packetsOut[queue->packetsOutCount++] = *packet;
}

int packetQueuePopIn(packetqueue_t *queue, packet_t *packet) {
  assertNotNull(queue, "Packet queue is NULL");
  assertNotNull(packet, "Packet is NULL");

  if(queue->packetsInCount == 0) return 0;

  *packet = queue->packetsIn[0];
  for(uint32_t i = 1; i < queue->packetsInCount; i++) {
    queue->packetsIn[i - 1] = queue->packetsIn[i];
  }
  queue->packetsInCount--;
  return 1;
}

int packetQueuePopOut(packetqueue_t *queue, packet_t *packet) {
  assertNotNull(queue, "Packet queue is NULL");
  assertNotNull(packet, "Packet is NULL");

  if(queue->packetsOutCount == 0) return 0;

  *packet = queue->packetsOut[0];
  for(uint32_t i = 1; i < queue->packetsOutCount; i++) {
    queue->packetsOut[i - 1] = queue->packetsOut[i];
  }
  queue->packetsOutCount--;
  return 1;
}